/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class ArrayCharacterIterator
  extends    AbstractCharacterIterator
  implements ICharacterIterator
{
  protected char[] values = CharacterIterator.defaultValues;
  protected int index = 0;
  
  public ArrayCharacterIterator()
  {
  }
  
  public ArrayCharacterIterator(char[] values)
  {
    if(values != null) this.values = values;
    else               this.values = new char[0];
  }
  
  public ArrayCharacterIterator(ArrayCharacterIterator r)
  {
    if(r != null) 
    {
      values = r.values;
      index  = r.index;
    }
    else values = new char[0];
  }

  public Object clone()
  {
    return new ArrayCharacterIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof ArrayCharacterIterator))
      return false;

    ArrayCharacterIterator obj = (ArrayCharacterIterator)o;

    if(index != obj.index) return false;
    if(values.length != obj.values.length) return false;
    
    for(int i = 0; i < values.length; i++)
      if(values[i] != obj.values[i]) return false;
    
    return true;   
  }
  
  public char value()
  {
    return values[index];
  }
  
  public boolean stop()
  {
    return index >= values.length;
  }
  
  public void shift()
  {
    index++;
  }

  public int hashCode()
  {
    int r = 113*index;
    for(int i = 0; i < values.length; i++)
    { 
      r *= 3;
      r += values[i];   
    }
    
    return r;
  }
}
