/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class NeighbourhoodDoubleIterator
  extends    AbstractDoubleIterator
  implements IDoubleIterator
{
  protected double point = 0;
  protected int offset = 0;
  protected int index  = 0;
  
  public NeighbourhoodDoubleIterator(double point, int offset)
  {
    if(offset < 0) offset = 0;
    if(offset > Integer.MAX_VALUE/2) offset = Integer.MAX_VALUE/2;
     
    this.point  = point;
    this.offset = offset;
  }

  public NeighbourhoodDoubleIterator(double point)
  {
    this(point, 0);
  }
  
  public NeighbourhoodDoubleIterator(NeighbourhoodDoubleIterator r)
  {
    this.point  = r.point;
    this.offset = r.offset;
    this.index  = r.index;
  }

  public Object clone()
  {
    return new NeighbourhoodDoubleIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof NeighbourhoodDoubleIterator))
      return false;

    NeighbourhoodDoubleIterator obj = (NeighbourhoodDoubleIterator)o;

    return    index  == obj.index 
           && (point  == obj.point || Double.isNaN(point) && Double.isNaN(obj.point))
           && offset == obj.offset;
  }
  
  public double value()
  {
    if(Double.isNaN(point))
      switch(index)
      {
        case 0:  return Double.NaN;
        case 1:  return Double.POSITIVE_INFINITY;
        case 2:  return Double.NEGATIVE_INFINITY;
        default: return ((index&1) == 0)?
                          DoubleIterator.shiftDouble(Double.NEGATIVE_INFINITY, index/2-1)
                        : DoubleIterator.shiftDouble(Double.POSITIVE_INFINITY, -index/2);
      }
    else return DoubleIterator.shiftDouble(point, (index%2 == 0)?(index/2):(-(index/2+1)));
  }
  
  public boolean stop()
  {
    return index > 2*offset;
  }
  
  public void shift()
  {
    index++;
    
    if(index == Integer.MAX_VALUE || Double.isNaN(point)) return;
    else if(point == Double.POSITIVE_INFINITY)
    {
      if((index&1) == 0 && index > 2) { index++; return; }
      else                                       return;
    }
    else if(point == Double.NEGATIVE_INFINITY)
    {
      if((index&1) != 0 && index > 1) { index++; return; } 
      else                                       return;
    }
    else
    {
      if(      index%2 == 0
            && point > DoubleIterator.shiftDouble(Double.MAX_VALUE, - index/2)
         ||    index%2 == 1
            && point < DoubleIterator.shiftDouble(-Double.MAX_VALUE, (index+1)/2)
        )
        index++;
    }
  }

  public int hashCode ()
  {
    return index + 3*(int)Double.doubleToLongBits(point) + 11*offset;
  }
}
