/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class NeighbourhoodIntegerIterator
  extends    AbstractIntegerIterator
  implements IIntegerIterator
{
  protected int point = 0;
  protected int offset = 0;
  protected int index  = 0;
  
  public NeighbourhoodIntegerIterator(int point, int offset)
  {
    if(offset < 0) offset = 0;
    if(offset > Integer.MAX_VALUE/2) offset = Integer.MAX_VALUE/2;
     
    this.point  = point;
    this.offset = offset;
  }

  public NeighbourhoodIntegerIterator(int point)
  {
    this(point, 0);
  }
  
  public NeighbourhoodIntegerIterator(NeighbourhoodIntegerIterator r)
  {
    this.point  = r.point;
    this.offset = r.offset;
    this.index  = r.index;
  }

  public Object clone()
  {
    return new NeighbourhoodIntegerIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof NeighbourhoodIntegerIterator))
      return false;

    NeighbourhoodIntegerIterator obj = (NeighbourhoodIntegerIterator)o;

    return    index  == obj.index 
           && point  == obj.point
           && offset == obj.offset;
  }
  
  public int value()
  {
    return (int)(point + ((index%2 == 0)?(index/2):(-(index/2+1))));
  }
  
  public boolean stop()
  {
    return index > 2*offset;
  }
  
  public void shift()
  {
    index++;
    
    if(index == Integer.MAX_VALUE) return;
    if(      index%2 == 0
           && point > Integer.MAX_VALUE - index/2
        ||    index%2 == 1
           && point < Integer.MIN_VALUE + (index+1)/2
      )
      index++;
  }

  public int hashCode ()
  {
    return index + 3*point + 11*offset;
  }
}
