/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class NeighbourhoodLongIterator
  extends    AbstractLongIterator
  implements ILongIterator
{
  protected long point = 0;
  protected long offset = 0;
  protected long index  = 0;
  
  public NeighbourhoodLongIterator(long point, long offset)
  {
    if(offset < 0) offset = 0;
    if(offset > Integer.MAX_VALUE/2) offset = Integer.MAX_VALUE/2;
     
    this.point  = point;
    this.offset = offset;
  }

  public NeighbourhoodLongIterator(long point)
  {
    this(point, 0);
  }
  
  public NeighbourhoodLongIterator(NeighbourhoodLongIterator r)
  {
    this.point  = r.point;
    this.offset = r.offset;
    this.index  = r.index;
  }

  public Object clone()
  {
    return new NeighbourhoodLongIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof NeighbourhoodLongIterator))
      return false;

    NeighbourhoodLongIterator obj = (NeighbourhoodLongIterator)o;

    return    index  == obj.index 
           && point  == obj.point
           && offset == obj.offset;
  }
  
  public long value()
  {
    if(index == Integer.MAX_VALUE) return (long)(point - Integer.MAX_VALUE/2 - 1);
    return (long)(point + ((index%2 == 0)?(index/2):(-(index+1)/2)));
  }
  
  public boolean stop()
  {
    return index > 2*offset;
  }
  
  public void shift()
  {
    index++;
    
    if(index == Integer.MAX_VALUE) return;
    if(      index%2 == 0
          && point > Long.MAX_VALUE - index/2
       ||    index%2 == 1
          && point < Long.MIN_VALUE + (index+1)/2
      )
      index++;
  }

  public int hashCode ()
  {
    return (int)(index + 3*point + 11*offset);
  }
}
