/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

import jatva.iterators.AbstractIterator;

public class ShortObjectIterator
  extends AbstractIterator
  implements jatva.iterators.Iterator
{
  public static Short[] defaultValues = {
     null
   , new Short((short)(Short.MIN_VALUE + 1))
   , new Short((short)0)
   , null
  };
  static
  {
    defaultValues[3] = defaultValues[1];
  }

  protected Short[] values = defaultValues;
  protected IShortIterator it = new ArrayShortIterator();
  protected int index = 0;
  
  public ShortObjectIterator()
  {
  }

  public ShortObjectIterator(Short[] values)
  {
    if(values == null) throw new IllegalArgumentException("Argument should not be null");
    this.values = new Short[values.length];
    it = null;
    
    for(int i = 0; i < values.length; i++)
      this.values[i] = (values[i] == null)?null:(new Short(values[i].shortValue()));
  }

  public ShortObjectIterator(IShortIterator it)
  {
    if(it == null) throw new IllegalArgumentException("Argument should not be null");
    values = null;
    this.it = (IShortIterator)it.clone();
  }
  
  public ShortObjectIterator(Short[] values, IShortIterator it)
  {
    if(values == null && it == null) throw new IllegalArgumentException("Both arguments should not be null");
    if(values != null)
    {
      this.values = new Short[values.length];
    
      for(int i = 0; i < values.length; i++)
        this.values[i] = (values[i] == null)?null:(new Short(values[i].shortValue()));
    }
    if(it != null) this.it = (IShortIterator)it.clone();
  }
  
  public void setValueIterator(IShortIterator it)
  {
    if(it == null) throw new IllegalArgumentException("Argument should not be null");
    this.it = (IShortIterator)it.clone();
  }
  
  public void setArray(Short[] values)
  {
    if(values == null) throw new IllegalArgumentException("Argument should not be null");
    this.values = new Short[values.length];
    
    for(int i = 0; i < values.length; i++)
      this.values[i] = (values[i] == null)?null:new Short(values[i].shortValue());
  }

  public ShortObjectIterator(ShortObjectIterator r)
  {
    index = r.index;
    it = (r.it == null)?null:(IShortIterator)r.it.clone();
    if(r.values == null) { values = null; return; }
    values = new Short[r.values.length];
    
    for(int i = 0; i < values.length; i++)
      values[i] = (r.values[i] == null)?null:(new Short(r.values[i].shortValue()));
  }
  
  public Object clone()
  {
    return new ShortObjectIterator(this);
  }
  
  public boolean equals(Object o) 
  {
    if(!(o instanceof ShortObjectIterator))
      return false;

    ShortObjectIterator obj = (ShortObjectIterator)o;

    if(index != obj.index) return false;
    if(   it == null && obj.it != null
       || it != null && !it.equals(obj.it)
      ) return false;
    if(values == null) return (obj.values == null);
    if(values != null && obj.values == null) return false;
    if(values.length != obj.values.length) return false;
    
    for(int i = 0; i < values.length; i++)
      if(   values[i] == null && obj.values[i] != null
         || values[i] != null && !values[i].equals(obj.values[i])
        ) return false;
    
    return true;   
  }

  public Short value()
  {
    if(values != null && index < values.length) return values[index];
    else if(it != null)       return new Short(it.value());
    else                      return null;
  }
  
  public boolean stop()
  {
    if(it == null)              return (index >= values.length);
    else if(values == null)     return it.stop();
    else                        return index >= values.length && it.stop();
  }
  
  public void shift()
  {
    if(values != null && index < values.length) index++;
    else if(it != null)       it.next();
  }
  
  public String toString()
  {
    if(stop()) return jatva.iterators.Iterator.EXHAUSTED;
    return "" + value();
  }

  public Object next()
  {
    Object r = value();
    shift();
    return r;    
  }

  public int hashCode ()
  {
    int r = index;
    if(values != null)
      for(int i = 0; i < values.length; i++)
        r += values[i].byteValue()*i;
    
    if(it != null)
      r += it.hashCode();
    
    return r;
  }
}
