/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;
import jatva.lang.synchronization.TestManager;

/**
 * Provides handy accessors to attributes of model events.
 *
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public class ModelEventHelper
{
    @SuppressWarnings( "unchecked" )
    public static <T> T getArgument( ModelEvent e, String arg_name )
    {
        int arg_number = -1;
        for( String par_name : e.getOperation().getParNames() )
        {
            arg_number ++;
            if( par_name.equals( arg_name ) )
                break;
        }
        if( arg_number == -1 )
        {
            throw new IllegalArgumentException(   "no arg named "
                                                + "'" + arg_name + "'"
                                                + " found in signature "
                                                + "'" + e.getOperation().getPrettySignature() + "'"
                                              );
        }

        return (T)e.getArguments()[ arg_number ];
    }

    @SuppressWarnings( "unchecked" )
    public static <T> T getArgument( ModelEvent e, int arg_num )
    {
        return (T)e.getArguments()[ arg_num ];
    }

    public static String getOperationName( ModelEvent me )
    {
        return me.getOperation().getMethod().getName();
    }

    public static String getPrettySignature( ModelEvent me )
    {
        return me.getOperation().getPrettySignature();
    }

    public static ModelEvent getCurrentEvent()
    {
        return TestManager.getRegistrar().getCurrentEvent();
    }
}
