/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;

import jatva.lang.synchronization.Test;
import jatva.lang.synchronization.TestManager;
import jatva.util.Utils;

public class TestException extends Error
{
    protected Throwable _cause = null;

    protected boolean _isCritical = false;

    @Override
    public Throwable getCause()
    {
        return _cause;
    }

    public boolean isCritical()
    {
        return _isCritical;
    }

    protected TestException( String info
                           , Throwable error
                           , boolean isCritical
                           , boolean isInternal
                           , boolean isTraced
                           , String[] tags
                           , String[] values
                           )
    {
        super( info );
        _cause = error;
        _isCritical = isCritical;

        TestManager.getRegistrar().testException( this
                                                , info
                                                , error
                                                , isCritical
                                                , isInternal
                                                , isTraced
                                                , tags
                                                , values
                                                );
    }

    public TestException( String info
                        , Throwable error
                        , boolean isCritical
                        , String[] tags
                        , String[] values
                        )
    {
        this( info, error, isCritical, false, true, tags, values );
    }

    protected TestException( String info
                           , boolean isCritical
                           , String[] tags
                           , String[] values
                           )
    {
        this( info, null, isCritical, false, true, tags, values );
    }

    public TestException( String info
                        , Throwable error
                        , String[] tags
                        , String[] values
                        )
    {
        this( info, error, false, false, true, tags, values );
    }

    public TestException( String info, String[] tags, String[] values )
    {
        this( info, null, false, false, true, tags, values );
    }

    protected static String formatModelObject( ModelObject mo )
    {
        if( TestManager.getRegistrar().getStatus() == Test.CHECKING )
        {
            // we're checking oracles in serialization
            // no 'model object' tags available.
            // so we should print model object deeply
            return Utils.getDeepString( mo );
        } else {
            // we're executing either in synchronous mode
            // or asynchronous one. should print trivial string
            return Utils.getSimpleString( mo );
        }
    }
}
