/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.util;

import jatva.lang.InsufficientAccessibilityError;
import jatva.lang.TranslationError;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/**
 *
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public class MethodInfo extends OperationInfo
{
    protected Method method;

    public MethodInfo( String extendedReflectSignature )
    {
        super( extendedReflectSignature );
    }

    public Method getMethod()
    {
        return method;
    }

    @Override
    protected final Class<?>[] obtainExecutableMember()
    {
        method = Utils.getMethodByStringPrefix( reflectSignature );

        if( method == null )
            throw new TranslationError( "Invalid method signature"
                                      , new String[]{ "method signature" }
                                      , new String[]{ reflectSignature }
                                      );

        try
        {
            method.setAccessible( true );
        }
        catch( SecurityException e )
        {
          throw new InsufficientAccessibilityError( "Insufficient rights to call implementation method through reflection"
                                                  , e
                                                  , new String[]{ "method"}
                                                  , new String[]{ reflectSignature }
                                                  );
        }

        this.declaringClass = Utils.prettifyClass( method.getDeclaringClass() );
        this.returnType = Utils.prettifyClass( method.getReturnType() );

        return method.getParameterTypes();
    }

    @Override
    protected final void buildPrettySignature()
    {
        boolean is_static = ( method.getModifiers() & Modifier.STATIC ) != 0;

        StringBuffer res = new StringBuffer();

        if( is_static )
            res.append( "static " );
        res.append( returnType );
        res.append( ' ' );
        res.append( declaringClass );
        res.append( '.' );
        res.append( method.getName() );
        appendParTypes( res );
        this.prettySignature = res.toString();
    }
}
